﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Security.Cryptography;
using System.Text;
using ININ.IntegrationServer.Plugins;
using ININ.IntegrationServer.Plugins.Actions;
using ININ.IntegrationServer.Plugins.Domain;
using ININ.IntegrationServer.Plugins.Domain.Model;
using ININ.IntegrationServer.Plugins.Domain.Params;
using SugarCRMPlugin.Properties;
using SugarCRMPlugin.SugarCRM;

namespace SugarCRMPlugin
{
    public class SugarCrmPlugin : CrmPlugin
    {
        public override string CompanyName
        {
            get { return "MyCompany"; }
        }
        public override string DomainName
        {
            get { return "MyDomain"; }
        }
        public override string PluginName
        {
            get { return "SugarCRM"; }
        }

        private string _crmSessionId = String.Empty;
        private readonly SugarCrmConfigurationSettings _sugarCrmConfigurationSettings;
        private readonly sugarsoap _sugarClient = new sugarsoap();

        public SugarCrmPlugin(IConfigurationService configurationService)
        {
            _sugarCrmConfigurationSettings = configurationService.GetConfiguration<SugarCrmConfigurationSettings>();
            LoginToCrm(_sugarCrmConfigurationSettings.Username, _sugarCrmConfigurationSettings.Password);
        }

        private void LoginToCrm(string username, string password)
        {
            _sugarClient.Timeout = 900000;
            _sugarClient.Url = Settings.Default.SugarCRMPlugin_SugarCRM_sugarsoap;

            // Create Authentication Object & Populate Credentials
            var userAuth = new user_auth { user_name = username, password = GetMd5(password) };

            // Authenticate
            var loginList = new name_value[0];
            var loginResult = _sugarClient.login(userAuth, "CrmPluginTest", loginList);

            // Get Session Id
            _crmSessionId = loginResult.id;
        }
        static private string GetMd5(string textString)
        {
            var md5 = MD5.Create();
            var inputBuffer = Encoding.ASCII.GetBytes(textString);
            var outputBuffer = md5.ComputeHash(inputBuffer);

            var builder = new StringBuilder(outputBuffer.Length);
            for (var i = 0; i < outputBuffer.Length; i++)
            {
                builder.Append(outputBuffer[i].ToString("X2"));
            }
            return builder.ToString();
        }

        public override ISyncResult<IActionData> GetContactByPhoneNumber(IActionParams actionParams, GetContactByPhoneNumberRequestData data)
        {
            // Return an error if no phone number is provided
            if (String.IsNullOrWhiteSpace(data.PhoneNumber))
            {
                return ResponseFactory.CreateErrorResponse(new ErrorResponseData("Required field 'PhoneNumber' was not provided."));
            }

            // This will be used to return data to the IVR
            IActionData responseData;

            // Call the Sugar Web Service. Field names can be found here: http://apidocs.sugarcrm.com/schema/6.3.0/pro/tables/contacts.html
            var contacts = _sugarClient.get_entry_list(_crmSessionId, "Contacts", String.Format("contacts.phone_work='{0}'", data.PhoneNumber), String.Empty, 0, null, null, 10, 0, false);
            if (contacts.entry_list.Any())
            {
                var apiContact = contacts.entry_list[0];

                // Create a new contact and populate it with some of the information returned by SugarCRM
                var contact = new Contact
                {
                    Id = apiContact.id,
                    FullName = GetValueFromNameValueList("name", apiContact.name_value_list),
                    FirstName = GetValueFromNameValueList("first_name", apiContact.name_value_list),
                    LastName = GetValueFromNameValueList("last_name", apiContact.name_value_list),
                    Address = new Address
                    {
                        Line1 = GetValueFromNameValueList("primary_address_street", apiContact.name_value_list),
                        City = GetValueFromNameValueList("primary_address_city", apiContact.name_value_list),
                        Country = GetValueFromNameValueList("primary_address_country", apiContact.name_value_list),
                        PostalCode = GetValueFromNameValueList("primary_address_postalcode", apiContact.name_value_list)
                    }
                };

                // Build the response data
                responseData = new GetContactByPhoneNumberResponseData { Contact = contact };
            }
            else
            {
                responseData = new NoResultResponseData();
            }

            // Return the response data
            return ResponseFactory.CreateResponse(responseData);
        }
        private static string GetValueFromNameValueList(string key, IEnumerable<name_value> nameValues)
        {
            return nameValues.Where(nv => nv.name == key).ToArray()[0].value;
        }
    }
}
